@@ -1,6 +1,5 @@
 
 --- Fetch back-end for retrieving sources from GIT.
---module("luarocks.fetch.git", package.seeall)
 local git = {}
 
 local unpack = unpack or table.unpack
@@ -40,6 +39,11 @@ function git.get_sources(rockspec, extract, dest_dir, depth)
    -- Strip off .git from base name if present
    module = module:gsub("%.git$", "")
 
+   local ok, err_msg = fs.is_tool_available(git_cmd, "Git")
+   if not ok then
+      return nil, err_msg
+   end
+
    local store_dir
    if not dest_dir then
       store_dir = fs.make_temp_dir(name_version)
@@ -63,7 +67,7 @@ function git.get_sources(rockspec, extract, dest_dir, depth)
       if git_can_clone_by_tag(git_cmd) then
          -- The argument to `--branch` can actually be a branch or a tag as of
          -- Git 1.7.10.
-         table.insert(command, 4, "--branch=" .. tag_or_branch)
+         table.insert(command, 3, "--branch=" .. tag_or_branch)
       end
    end
    if not fs.execute(unpack(command)) then
