@@ -1,6 +1,6 @@
 
 --- Fetch back-end for retrieving sources from local Git repositories.
-module("luarocks.fetch.git_file", package.seeall)
+local git_file = {}
 
 local git = require("luarocks.fetch.git")
 
@@ -11,7 +11,9 @@ local git = require("luarocks.fetch.git")
 -- @return (string, string) or (nil, string): The absolute pathname of
 -- the fetched source tarball and the temporary directory created to
 -- store it; or nil and an error message.
-function get_sources(rockspec, extract, dest_dir)
+function git_file.get_sources(rockspec, extract, dest_dir)
    rockspec.source.url = rockspec.source.url:gsub("^git.file://", "")
    return git.get_sources(rockspec, extract, dest_dir)
 end
+
+return git_file
