@@ -1,76 +1,19 @@
 
 --- fs operations implemented with third-party tools for Unix platform abstractions.
---module("luarocks.fs.unix.tools", package.seeall)
 local tools = {}
 
 local fs = require("luarocks.fs")
 local dir = require("luarocks.dir")
 local cfg = require("luarocks.cfg")
 
-local dir_stack = {}
-
 local vars = cfg.variables
 
-local function command_at(directory, cmd)
-   return "cd " .. fs.Q(directory) .. " && " .. cmd
-end
-
---- Obtain current directory.
--- Uses the module's internal directory stack.
--- @return string: the absolute pathname of the current directory.
-function tools.current_dir()
-   local current = cfg.cache_pwd
-   if not current then
-      local pipe = io.popen(fs.Q(vars.PWD))
-      current = pipe:read("*l")
-      pipe:close()
-      cfg.cache_pwd = current
-   end
-   for _, directory in ipairs(dir_stack) do
-      current = fs.absolute_name(directory, current)
-   end
-   return current
-end
-
---- Run the given command.
--- The command is executed in the current directory in the directory stack.
--- @param cmd string: No quoting/escaping is applied to the command.
--- @return boolean: true if command succeeds (status code 0), false
--- otherwise.
-function tools.execute_string(cmd)
-   local code, err = os.execute(command_at(fs.current_dir(), cmd))
-   if code == 0 or code == true then
-      return true
-   else
-      return false
-   end
-end
-
---- Change the current directory.
--- Uses the module's internal directory stack. This does not have exact
--- semantics of chdir, as it does not handle errors the same way,
--- but works well for our purposes for now.
--- @param directory string: The directory to switch to.
-function tools.change_dir(directory)
-   assert(type(directory) == "string")
-   if fs.is_dir(directory) then
-      table.insert(dir_stack, directory)
-      return true
-   end
-   return nil, "directory not found: "..directory
-end
-
---- Change directory to root.
--- Allows leaving a directory (e.g. for deleting it) in
--- a crossplatform way.
-function tools.change_dir_to_root()
-   table.insert(dir_stack, "/")
-end
-
---- Change working directory to the previous in the directory stack.
-function tools.pop_dir()
-   local directory = table.remove(dir_stack)
-   return directory ~= nil
+--- Adds prefix to command to make it run from a directory.
+-- @param directory string: Path to a directory.
+-- @param cmd string: A command-line string.
+-- @return string: The command-line with prefix.
+function tools.command_at(directory, cmd)
+   return "cd " .. fs.Q(fs.absolute_name(directory)) .. " && " .. cmd
 end
 
 --- Create a directory if it does not already exist.
@@ -153,20 +96,6 @@ function tools.delete(arg)
    fs.execute_quiet(vars.RM, "-rf", arg)
 end
 
---- Internal implementation function for fs.dir.
--- Yields a filename on each iteration.
--- @param at string: directory to list
--- @return nil
-function tools.dir_iterator(at)
-   local pipe = io.popen(command_at(at, vars.LS))
-   for file in pipe:lines() do
-      if file ~= "." and file ~= ".." then
-         coroutine.yield(file)
-      end
-   end
-   pipe:close()
-end
-
 --- Recursively scan the contents of a directory.
 -- @param at string or nil: directory to scan (will be the current
 -- directory if none is given).
@@ -181,7 +110,7 @@ function tools.find(at)
       return {}
    end
    local result = {}
-   local pipe = io.popen(command_at(at, vars.FIND.." * 2>/dev/null"))
+   local pipe = io.popen(fs.command_at(at, fs.quiet_stderr(vars.FIND.." *")))
    for file in pipe:lines() do
       table.insert(result, file)
    end
@@ -230,51 +159,6 @@ function tools.is_file(file)
    return fs.execute(vars.TEST, "-f", file)
 end
 
---- Download a remote file.
--- @param url string: URL to be fetched.
--- @param filename string or nil: this function attempts to detect the
--- resulting local filename of the remote file as the basename of the URL;
--- if that is not correct (due to a redirection, for example), the local
--- filename can be given explicitly as this second argument.
--- @return (boolean, string): true and the filename on success,
--- false and the error message on failure.
-function tools.use_downloader(url, filename, cache)
-   assert(type(url) == "string")
-   assert(type(filename) == "string" or not filename)
-
-   filename = fs.absolute_name(filename or dir.base_name(url))
-
-   local ok
-   if cfg.downloader == "wget" then
-      local wget_cmd = fs.Q(vars.WGET).." --no-check-certificate --no-cache --user-agent='"..cfg.user_agent.." via wget' --quiet "
-      if cfg.connection_timeout and cfg.connection_timeout > 0 then
-        wget_cmd = wget_cmd .. "--timeout="..tonumber(cfg.connection_timeout).." --tries=1 " 
-      end
-      if cache then
-         -- --timestamping is incompatible with --output-document,
-         -- but that's not a problem for our use cases.
-         fs.change_dir(dir.dir_name(filename))
-         ok = fs.execute_quiet(wget_cmd.." --timestamping ", url)
-         fs.pop_dir()
-      elseif filename then
-         ok = fs.execute_quiet(wget_cmd.." --output-document ", filename, url)
-      else
-         ok = fs.execute_quiet(wget_cmd, url)
-      end
-   elseif cfg.downloader == "curl" then
-      local curl_cmd = fs.Q(vars.CURL).." -f -k -L --user-agent '"..cfg.user_agent.." via curl' "
-      if cfg.connection_timeout and cfg.connection_timeout > 0 then
-        curl_cmd = curl_cmd .. "--connect-timeout "..tonumber(cfg.connection_timeout).." " 
-      end
-      ok = fs.execute_string(curl_cmd..fs.Q(url).." 2> /dev/null 1> "..fs.Q(filename))
-   end
-   if ok then
-      return true, filename
-   else
-      return false
-   end
-end
-
 function tools.chmod(pathname, mode)
    if mode then 
       return fs.execute(vars.CHMOD, mode, pathname)
@@ -283,12 +167,6 @@ function tools.chmod(pathname, mode)
    end
 end
 
---- Apply a patch.
--- @param patchname string: The filename of the patch.
-function tools.apply_patch(patchname)
-   return fs.execute(vars.PATCH.." -p1 -f -i ", patchname)
-end
-
 --- Unpack an archive.
 -- Extract the contents of an archive, detecting its format by
 -- filename extension.
@@ -297,19 +175,24 @@ end
 function tools.unpack_archive(archive)
    assert(type(archive) == "string")
 
+   local pipe_to_tar = " | "..vars.TAR.." -xf -"
+
+   if not cfg.verbose then
+      pipe_to_tar = " 2> /dev/null"..fs.quiet(pipe_to_tar)
+   end
+
    local ok
    if archive:match("%.tar%.gz$") or archive:match("%.tgz$") then
-         ok = fs.execute_string(vars.GUNZIP.." -c "..archive.."|"..vars.TAR.." -xf -")
+      ok = fs.execute_string(vars.GUNZIP.." -c "..fs.Q(archive)..pipe_to_tar)
    elseif archive:match("%.tar%.bz2$") then
-         ok = fs.execute_string(vars.BUNZIP2.." -c "..archive.."|tar -xf -")
+      ok = fs.execute_string(vars.BUNZIP2.." -c "..fs.Q(archive)..pipe_to_tar)
    elseif archive:match("%.zip$") then
-      ok = fs.execute(vars.UNZIP, archive)
+      ok = fs.execute_quiet(vars.UNZIP, archive)
    elseif archive:match("%.lua$") or archive:match("%.c$") then
       -- Ignore .lua and .c files; they don't need to be extracted.
       return true
    else
-      local ext = archive:match(".*(%..*)")
-      return false, "Unrecognized filename extension "..(ext or "")
+      return false, "Couldn't extract archive "..archive..": unrecognized filename extension"
    end
    if not ok then
       return false, "Failed extracting "..archive
@@ -317,28 +200,6 @@ function tools.unpack_archive(archive)
    return true
 end
 
-local md5_cmd = {
-   md5sum = vars.MD5SUM,
-   openssl = vars.OPENSSL.." md5",
-   md5 = vars.MD5,
-}
-
---- Get the MD5 checksum for a file.
--- @param file string: The file to be computed.
--- @return string: The MD5 checksum
-function tools.get_md5(file)
-   local cmd = md5_cmd[cfg.md5checker]
-   if not cmd then return nil, "no MD5 checker command configured" end
-   local pipe = io.popen(cmd.." "..fs.Q(fs.absolute_name(file)))
-   local computed = pipe:read("*a")
-   pipe:close()
-   if computed then
-      computed = computed:match("("..("%x"):rep(32)..")")
-   end
-   if computed then return computed end
-   return nil, "Failed to compute MD5 hash for file "..tostring(fs.absolute_name(file))
-end
-
 function tools.get_permissions(filename)
    local pipe = io.popen(vars.STAT.." "..vars.STATFLAG.." "..fs.Q(filename))
    local ret = pipe:read("*l")
@@ -350,4 +211,27 @@ function tools.browser(url)
    return fs.execute(cfg.web_browser, url)
 end
 
+function tools.set_time(file, time)
+   file = dir.normalize(file)
+   return fs.execute(vars.TOUCH, "-d", "@"..tostring(time), file)
+end
+
+--- Create a temporary directory.
+-- @param name string: name pattern to use for avoiding conflicts
+-- when creating temporary directory.
+-- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.
+function tools.make_temp_dir(name)
+   assert(type(name) == "string")
+   name = dir.normalize(name)
+
+   local template = (os.getenv("TMPDIR") or "/tmp") .. "/luarocks_" .. name:gsub(dir.separator, "_") .. "-XXXXXX"
+   local pipe = io.popen(vars.MKTEMP.." -d "..fs.Q(template))
+   local dirname = pipe:read("*l")
+   pipe:close()
+   if dirname and dirname:match("^/") then
+      return dirname
+   end
+   return nil, "Failed to create temporary directory "..tostring(dirname)
+end
+
 return tools
